@@ -19,12 +19,12 @@
 #ifndef INCLUDED_COM_SUN_STAR_UNO_REFERENCE_H
 #define INCLUDED_COM_SUN_STAR_UNO_REFERENCE_H
 
-#include <sal/config.h>
+#include "sal/config.h"
 
 #include <cassert>
 #include <cstddef>
 
-#include <rtl/alloc.h>
+#include "rtl/alloc.h"
 
 namespace com
 {
@@ -83,16 +83,25 @@ public:
 
         @return UNacquired interface pointer
     */
-    inline XInterface * SAL_CALL get() const
+    XInterface * SAL_CALL get() const
         { return _pInterface; }
 
     /** Checks if reference is null.
 
         @return true if reference acquires an interface, i.e. true if it is not null
     */
-    inline bool SAL_CALL is() const
+    bool SAL_CALL is() const
         { return (NULL != _pInterface); }
 
+#if defined LIBO_INTERNAL_ONLY
+    /** Checks if reference is null.
+
+        @return true if reference acquires an interface, i.e. true if it is not null
+    */
+    explicit operator bool() const
+        { return is(); }
+#endif
+
     /** Equality operator: compares two interfaces
         Checks if both references are null or refer to the same object.
 
@@ -247,7 +256,7 @@ class SAL_DLLPUBLIC_RTTI Reference : public BaseReference
         principle, this is not guaranteed to work.  In practice, it seems to
         work on all supported platforms.
     */
-    static inline interface_type * castFromXInterface(XInterface * p) {
+    static interface_type * castFromXInterface(XInterface * p) {
         return static_cast< interface_type * >(static_cast< void * >(p));
     }
 
@@ -262,26 +271,26 @@ class SAL_DLLPUBLIC_RTTI Reference : public BaseReference
         principle, this is not guaranteed to work.  In practice, it seems to
         work on all supported platforms.
     */
-    static inline XInterface * castToXInterface(interface_type * p) {
+    static XInterface * castToXInterface(interface_type * p) {
         return static_cast< XInterface * >(static_cast< void * >(p));
     }
 
 public:
     /// @cond INTERNAL
     // these are here to force memory de/allocation to sal lib.
-    inline static void * SAL_CALL operator new ( ::size_t nSize )
+    static void * SAL_CALL operator new ( ::size_t nSize )
         { return ::rtl_allocateMemory( nSize ); }
-    inline static void SAL_CALL operator delete ( void * pMem )
+    static void SAL_CALL operator delete ( void * pMem )
         { ::rtl_freeMemory( pMem ); }
-    inline static void * SAL_CALL operator new ( ::size_t, void * pMem )
+    static void * SAL_CALL operator new ( ::size_t, void * pMem )
         { return pMem; }
-    inline static void SAL_CALL operator delete ( void *, void * )
+    static void SAL_CALL operator delete ( void *, void * )
         {}
     /// @endcond
 
     /** Destructor: Releases interface if set.
     */
-    inline ~Reference();
+    inline ~Reference() COVERITY_NOEXCEPT_FALSE;
 
     /** Default Constructor: Sets null reference.
     */
@@ -293,6 +302,14 @@ public:
     */
     inline Reference( const Reference< interface_type > & rRef );
 
+#if defined LIBO_INTERNAL_ONLY
+    /** Move constructor
+
+        @param rRef another reference
+    */
+    inline Reference( Reference< interface_type > && rRef );
+#endif
+
     /** Up-casting conversion constructor: Copies interface reference.
 
         Does not work for up-casts to ambiguous bases.  For the special case of
@@ -325,7 +342,7 @@ public:
         @param pInterface another reference
         @param dummy UNO_REF_NO_ACQUIRE to force obvious distinction to other constructors
     */
-    inline Reference( interface_type * pInterface, UnoReference_NoAcquire dummy );
+    inline SAL_DEPRECATED("use SAL_NO_ACQUIRE version") Reference( interface_type * pInterface, UnoReference_NoAcquire dummy );
 
     /** Constructor: Queries given interface for reference interface type (interface_type).
 
@@ -353,6 +370,12 @@ public:
                      to other constructors
     */
     inline Reference( const BaseReference & rRef, UnoReference_QueryThrow dummy );
+#ifdef LIBO_INTERNAL_ONLY
+    /**
+        Prevent code from calling the QUERY_THROW constructor, when they meant to use the SET_THROW constructor.
+    */
+    Reference( const Reference< interface_type > & rRef, UnoReference_QueryThrow dummy ) = delete;
+#endif
     /** Constructor: Queries given interface for reference interface type (interface_type).
         Throws a RuntimeException if the demanded interface cannot be queried.
 
@@ -392,23 +415,33 @@ public:
         any interface must be derived from com.sun.star.uno.XInterface.
         This a useful direct cast possibility.
     */
-    inline SAL_CALL operator const Reference< XInterface > & () const
+    SAL_CALL operator const Reference< XInterface > & () const
         { return * reinterpret_cast< const Reference< XInterface > * >( this ); }
 
     /** Dereference operator: Used to call interface methods.
 
         @return UNacquired interface pointer
     */
-    inline interface_type * SAL_CALL operator -> () const {
-        assert(_pInterface != 0);
+    interface_type * SAL_CALL operator -> () const {
+        assert(_pInterface != NULL);
         return castFromXInterface(_pInterface);
     }
 
+    /** Indirection operator.
+
+        @since LibreOffice 6.3
+        @return UNacquired interface reference
+    */
+    interface_type & SAL_CALL operator * () const {
+        assert(_pInterface != NULL);
+        return *castFromXInterface(_pInterface);
+    }
+
     /** Gets interface pointer. This call does not acquire the interface.
 
         @return UNacquired interface pointer
     */
-    inline interface_type * SAL_CALL get() const
+    interface_type * SAL_CALL get() const
         { return castFromXInterface(_pInterface); }
 
     /** Clears reference, i.e. releases interface. Reference is null after clear() call.
@@ -443,7 +476,7 @@ public:
         @param dummy UNO_REF_NO_ACQUIRE to force obvious distinction to set methods
         @return true, if non-null interface was set
     */
-    inline bool SAL_CALL set( interface_type * pInterface, UnoReference_NoAcquire dummy);
+    inline SAL_DEPRECATED("use SAL_NO_ACQUIRE version") bool SAL_CALL set( interface_type * pInterface, UnoReference_NoAcquire dummy);
 
     /** Queries given interface for reference interface type (interface_type) and sets it.
         An interface already set will be released.
@@ -493,6 +526,12 @@ public:
                to set methods
     */
     inline void SAL_CALL set( const BaseReference & rRef, UnoReference_QueryThrow dummy );
+#ifdef LIBO_INTERNAL_ONLY
+    /**
+        Prevent code from calling the QUERY_THROW version, when they meant to use the SET_THROW version.
+    */
+    void set( const Reference< interface_type > & rRef, UnoReference_QueryThrow dummy ) = delete;
+#endif
 
     /** Queries given any for reference interface type (interface_type) and
         sets it.  An interface already set will be released.
@@ -540,31 +579,29 @@ public:
         @return this reference
     */
     inline Reference< interface_type > & SAL_CALL operator = ( const Reference< interface_type > & rRef );
+#if defined LIBO_INTERNAL_ONLY
+    /** Assignment move operator: Acquires given interface reference and sets reference.
+        An interface already set will be released.
 
+        @param rRef an interface reference
+        @return this reference
+    */
+    inline Reference< interface_type > & SAL_CALL operator = ( Reference< interface_type > && rRef );
+#endif
     /** Queries given interface reference for type interface_type.
 
         @param rRef interface reference
         @return interface reference of demanded type (may be null)
     */
-    inline static SAL_WARN_UNUSED_RESULT Reference< interface_type > SAL_CALL query( const BaseReference & rRef );
+    SAL_WARN_UNUSED_RESULT inline static Reference< interface_type > SAL_CALL query( const BaseReference & rRef );
     /** Queries given interface for type interface_type.
 
         @param pInterface interface pointer
         @return interface reference of demanded type (may be null)
     */
-    inline static SAL_WARN_UNUSED_RESULT Reference< interface_type > SAL_CALL query( XInterface * pInterface );
+    SAL_WARN_UNUSED_RESULT inline static Reference< interface_type > SAL_CALL query( XInterface * pInterface );
 };
 
-/// @cond INTERNAL
-/** Enables boost::mem_fn and boost::bind to recognize Reference.
-*/
-template <typename T>
-inline T * get_pointer( Reference<T> const& r )
-{
-    return r.get();
-}
-/// @endcond
-
 }
 }
 }
